# w.c.s. - web application for online forms
# Copyright (C) 2005-2014  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import fnmatch
import json
import os
import sys
import tempfile
import urllib2

from qommon.ctl import Command, make_option
from qommon.storage import atomic_write


class CmdCheckHobos(Command):
    name = 'check-hobos'

    def __init__(self):
        Command.__init__(self, [
                make_option('--patterns', metavar='FILENAME', action='store',
                            dest='patterns'),
                ])

    def is_for_us(self, service):
        # This function checks if the requested service is to be hosted
        # on this server, it matches against a set of patterns taken from
        # a file such formatted like this:
        #  | *.au-quotidien.com (globs are accepted)
        #  | ! *.dev.au-quotidien.com (possibility to prefix with ! to exclude)
        # and return True only if appropriate.
        if not self.patterns:
            return True
        parsed_url = urllib2.urlparse.urlsplit(service.get('base_url'))
        netloc = parsed_url.netloc
        match = False
        for pattern in self.patterns:
            if pattern.startswith('#'):
                # ignore comments
                continue
            if pattern.startswith('!'):
                if fnmatch.fnmatch(netloc, pattern[1:].strip()):
                    match = False
                    break
            else:
                if fnmatch.fnmatch(netloc, pattern.strip()):
                    match = True
        return match

    def execute(self, base_options, sub_options, args):
        import publisher

        publisher.WcsPublisher.configure(self.config, sub_options.extra)
        pub = publisher.WcsPublisher.create_publisher()
        self.patterns = []
        if sub_options.patterns:
            self.patterns = file(sub_options.patterns).readlines()

        global_app_dir = pub.app_dir

        hobos = []
        for arg in args:
            try:
                hobos.extend(json.load(urllib2.urlopen(arg + '/hobos.json')))
            except (urllib2.URLError, urllib2.HTTPError), e:
                print >> sys.stderr, 'failed to get URL', arg, e
                continue

        for hobo in hobos:
            try:
                all_services = json.load(
                        urllib2.urlopen(hobo + 'environment/installed_services.json'))
            except (urllib2.URLError, urllib2.HTTPError), e:
                print >> sys.stderr, 'failed to get URL', hobo, e
                continue
            services = [x for x in all_services if x.get('service-id') == 'wcs' and self.is_for_us(x)]

            # initialize all instances of w.c.s.
            for service in services:
                parsed_url = urllib2.urlparse.urlsplit(service.get('base_url'))
                instance_path = parsed_url.netloc
                if parsed_url.path:
                    instance_path = '%s+' % parsed_url.path.replace('/', '+')
                pub.app_dir = os.path.join(global_app_dir, instance_path)
                if not os.path.exists(pub.app_dir):
                    print 'initializing instance in', pub.app_dir
                    os.mkdir(pub.app_dir)
                    pub.initialize_app_dir()
                else:
                    pub.set_config()

                if not pub.cfg.get('misc'):
                    pub.cfg['misc'] = {}
                pub.cfg['misc']['sitename'] = service.get('title').encode('utf-8')

                # configure authentication methods
                idps = [x for x in all_services if x.get('service-id') == 'authentic']
                if not pub.cfg.get('identification'):
                    pub.cfg['identification'] = {}
                methods = pub.cfg['identification'].get('methods', [])
                if not methods:
                    methods = ['idp']
                elif not 'idp' in methods:
                    methods.append('idp')
                pub.cfg['identification']['methods'] = methods
                pub.write_cfg()

                if idps and not pub.cfg.get('sp', {}).get('publickey'):
                    from qommon.ident.idp import MethodAdminDirectory
                    if not pub.cfg.get('sp'):
                        pub.cfg['sp'] = {}
                    spconfig = pub.cfg['sp']
                    spconfig['base_url'] = str(service.get('base_url')) + '/liberty'
                    spconfig['saml2_base_url'] = str(service.get('base_url')) + '/saml'
                    spconfig['providerid'] = spconfig['base_url'] + '/metadata'
                    spconfig['saml2_providerid'] = spconfig['saml2_base_url'] + '/metadata'
                    MethodAdminDirectory().generate_rsa_keypair()

                for idp in idps:
                    metadata_url = '%s/idp/saml2/metadata' % idp['base_url']
                    try:
                        rfd = urllib2.urlopen(metadata_url)
                    except (urllib2.HTTPError, urllib2.URLError), e:
                        print >> sys.stderr, 'failed to get metadata URL', metadata_url, e
                        continue
                    except Exception, e:
                        print >> sys.stderr, 'failed to get metadata URL', metadata_url, e
                        continue

                    s = rfd.read()
                    (bfd, metadata_pathname) = tempfile.mkstemp('.metadata')
                    atomic_write(metadata_pathname, s)

                    from qommon.ident.idp import AdminIDPDir
                    admin_dir = AdminIDPDir()
                    key_provider_id = admin_dir.submit_new_remote(
                            metadata_pathname, None, metadata_url, None)
                    pub.cfg['idp'][key_provider_id]['admin-attributes'] = \
                            {'role': 'admin::%s' % str(service.get('slug'))}
                    pub.write_cfg()

                pub.write_cfg()

CmdCheckHobos.register()
